unit HEImgIndexEd;

//DELPHI 4.0 ONLY!!!
{$IFNDEF VER120}
error Delphi 4 only
{$ENDIF}

//Property Editor for 'ImageIndex'-property.
//  See: TMenuItem, TToolButton, TCustomAction, TTabSheet and TListColumn.
//Component Editor for components inherited from TCommonDialog.

//Author: Haakon Eines
//E-mail: haakon.eines@finale.no

//Version 1.0

interface

uses
  Windows, Classes, SysUtils, Menus, ComCtrls, ImgList,
  ActnList, Dialogs, DsgnIntf;

type
  //Now you can see the image...
  THEImageIndexEditor = class(TIntegerProperty)
  private
    EditMenu: TPopupMenu;
    procedure MenuClick(Sender: TObject);
  public
    destructor Destroy; override;
    procedure Initialize; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  //Now you can test and see your TPopupMenu in the Delphi IDE!
  THEAutoPopupEditor = class(TBoolProperty)
  private
    EditMenu: TPopupMenu;
    procedure MenuClick(Sender: TObject);
  public
    destructor Destroy; override;
    procedure Initialize; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  //Now you can test and see your Dialogs in the Delphi IDE!
  THECommonDialogComponentEditor = class(TComponentEditor)
  public
    function GetVerbCount: Integer; override;
    function GetVerb(Index: Integer): String; override;
    procedure Executeverb(Index: Integer); override;
  end;

  procedure Register;

implementation

type
  //Some Hacks...???
  TToolButtonHack = class(TToolButton);
  TListViewHack = class(TCustomListView);
  TCmDlgHack = class(TCommonDialog);

function GetMenuFromItem(MenuItem: TMenuItem): TMenu;
begin
  Result := nil;
  while (MenuItem <> nil) and (MenuItem.GetParentComponent <> nil) do
  begin
    if MenuItem.GetParentComponent is TMenu then
    begin
      Result := TMenu(MenuItem.GetParentComponent);
      Break;
    end;
    MenuItem := (MenuItem.GetParentComponent as TMenuItem);
  end;
end;

function AddMenuItem(AMenu: TMenu; ACaption: string; ImgIndex: Integer; ClickEvent: TNotifyEvent): TMenuItem;
begin
  Result := TMenuItem.Create(AMenu.Items);
  Result.Caption := ACaption;
  Result.OnClick := ClickEvent;
  Result.ImageIndex := ImgIndex;
  AMenu.Items.Add(Result);
end;

//---------------------------THEImageIndexEditor--------------------------------
destructor THEImageIndexEditor.Destroy;
begin
  if EditMenu <> nil then
    EditMenu.Free;
  EditMenu := nil;
  inherited Destroy;
end;

procedure THEImageIndexEditor.Initialize;
begin
  inherited Initialize;
  EditMenu := nil;
end;

procedure THEImageIndexEditor.MenuClick(Sender: TObject);
begin
  SetOrdValue(TMenuItem(Sender).ImageIndex);
end;

function THEImageIndexEditor.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog]
end;

procedure THEImageIndexEditor.Edit;
var
  i, ImgIndex: integer;
  FItem: TMenuItem;
  Cp: TPoint;
  Images: TCustomImageList;
  Menu: TMenu;
begin
  inherited Edit;
  Images := nil;
  //Get ImageList & ImageIndex
  if GetComponent(0) is TMenuItem then
  begin
    Menu := GetMenuFromItem(TMenuItem(GetComponent(0)));
    if Menu <> nil then
      Images := Menu.Images;
    ImgIndex := TMenuItem(GetComponent(0)).ImageIndex;
  end
  else if GetComponent(0) is TToolButton then
  begin
    Images := TToolButtonHack(GetComponent(0)).FToolBar.Images;
    ImgIndex := TToolButton(GetComponent(0)).ImageIndex;
  end
  else if GetComponent(0) is TCustomAction then
  begin
    Images := TCustomAction(GetComponent(0)).ActionList.Images;
    ImgIndex := TCustomAction(GetComponent(0)).ImageIndex;
  end
  else if GetComponent(0) is TTabSheet then
  begin
    Images := TTabSheet(GetComponent(0)).PageControl.Images;
    ImgIndex := TTabSheet(GetComponent(0)).ImageIndex;
  end
  else if GetComponent(0) is TListColumn then
  begin
    Images := TListViewHack(TListColumns(TListColumn(GetComponent(0)).Collection).Owner).SmallImages;
    if Images = nil then
      Images := TListViewHack(TListColumns(TListColumn(GetComponent(0)).Collection).Owner).LargeImages;
    ImgIndex := TListColumn(GetComponent(0)).ImageIndex;
  end
  else
    Exit;
  if Images = nil then Exit;

  if EditMenu <> nil then
    EditMenu.Free;

  //Create popupmenu
  EditMenu := TPopupMenu.Create(nil);
  //Assign ImageList
  EditMenu.Images := Images;
  //Add 'no-image'-item
  AddMenuItem(EditMenu, '-1', -1, MenuClick);
  //Populate with menuitems
  for i := 0 to EditMenu.Images.Count-1 do
  begin
    FItem := AddMenuItem(EditMenu, IntToStr(i), i, MenuClick);
    if ((i + 1) mod 20) = 0 then
      FItem.Break := mbBarBreak;
  end;
  //Check selected ImageIndex menuitem
  if ImgIndex = -1 then
    EditMenu.Items[0].Checked := True
  else if (ImgIndex+1 < EditMenu.Items.Count) then
    EditMenu.Items[ImgIndex+1].Checked := True;
  GetCursorPos(Cp);
  //Show popupmenu
  EditMenu.PopUp(Cp.X, Cp.Y);
end;


//---------------------------THEAutoPopupEditor---------------------------------
destructor THEAutoPopupEditor.Destroy;
begin
  if EditMenu <> nil then
    EditMenu.Free;
  EditMenu := nil;
  inherited Destroy;
end;

procedure THEAutoPopupEditor.Initialize;
begin
  inherited Initialize;
  EditMenu := nil;
end;

procedure THEAutoPopupEditor.MenuClick(Sender: TObject);
var
  Cp: TPoint;
begin
  case TMenuItem(Sender).Tag of
    0,1:
      SetOrdValue(TMenuItem(Sender).Tag);
    3: begin
         GetCursorPos(Cp);
         (GetComponent(0) as TPopupMenu).PopUp(Cp.X, Cp.Y);
       end;
  end;
end;

function THEAutoPopupEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

procedure THEAutoPopupEditor.Edit;
var
  FItem: TMenuItem;
  Cp: TPoint;

  procedure NewMenuItem(const ACaption: string; AChecked: Boolean; ATag: Longint);
  begin
    FItem := TMenuItem.Create(EditMenu.Items);
    FItem.Caption := ACaption;
    FItem.OnClick := MenuClick;
    FItem.Checked := AChecked;
    FItem.Tag := ATag;
    EditMenu.Items.Add(FItem);
  end;

begin
  if not(GetComponent(0) is TPopupMenu) then Exit;

  if EditMenu <> nil then
    EditMenu.Free;

  //Create popupmenu
  EditMenu := TPopupMenu.Create(nil);
  //Add items
  NewMenuItem('False', (GetOrdValue = 0), 0);
  NewMenuItem('True', (GetOrdValue <> 0), 1);
  NewMenuItem('-', False, 2);
  NewMenuItem('Test popupmenu...', False, 3);
  GetCursorPos(Cp);
  //Show popupmenu
  EditMenu.PopUp(Cp.X, Cp.Y);
end;

//---------------------- THECommonDialogComponentEditor ------------------------
function THECommonDialogComponentEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function THECommonDialogComponentEditor.GetVerb(Index : Integer): String;
begin
  if Index = 0 then
    Result := '&Execute ' + Component.ClassName
  else
    Result := inherited GetVerb(Index);
end;

procedure THECommonDialogComponentEditor.ExecuteVerb(Index: Integer);
begin
  if (Index = 0) and (Component is TCommonDialog) then
  begin
    TCmDlgHack(Component).SetDesigning(False);
    try
      TCmDlgHack(Component).Execute;
    finally
      TCmDlgHack(Component).SetDesigning(True);
    end;
    Designer.Modified
  end
end;


//------------------------Registration------------------------------------------
procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(Integer), TMenuItem, 'ImageIndex', THEImageIndexEditor);
  RegisterPropertyEditor(TypeInfo(Integer), TToolButton, 'ImageIndex', THEImageIndexEditor);
  RegisterPropertyEditor(TypeInfo(Integer), TCustomAction, 'ImageIndex', THEImageIndexEditor);
  RegisterPropertyEditor(TypeInfo(Integer), TTabSheet, 'ImageIndex', THEImageIndexEditor);
  RegisterPropertyEditor(TypeInfo(Integer), TListColumn, 'ImageIndex', THEImageIndexEditor);
  RegisterPropertyEditor(TypeInfo(Boolean), TPopupMenu, 'AutoPopup', THEAutoPopupEditor);
  RegisterComponentEditor(TCommonDialog,    THECommonDialogComponentEditor);
end;

end.
