unit TreeViewItemsForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ComCtrls, ExtCtrls, Buttons, ImgList;

type
  TTreeViewItemsDlg = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Button4: TButton;
    Cancel: TButton;
    ApplyBtn: TButton;
    Help: TButton;
    Panel3: TPanel;
    Panel4: TPanel;
    New: TButton;
    NewSub: TButton;
    Delete: TButton;
    Load: TButton;
    Button1: TButton;
    TreeView: TTreeView;
    Splitter1: TSplitter;
    Bevel1: TBevel;
    Panel5: TPanel;
    Panel6: TPanel;
    Label1: TLabel;
    AssignBtn: TBitBtn;
    ClearBtn: TButton;
    ImagesMenu: TPopupMenu;
    SelectedMenu: TPopupMenu;
    StateMenu: TPopupMenu;
    OpenDialog2: TOpenDialog;
    ImageBtn: TSpeedButton;
    Label5: TLabel;
    SelectedBtn: TSpeedButton;
    Label6: TLabel;
    Label7: TLabel;
    StateBtn: TSpeedButton;
    Bevel2: TBevel;
    Text: TEdit;
    TreeMenu: TPopupMenu;
    ImagePaint: TPaintBox;
    SelectedPaint: TPaintBox;
    StatePaint: TPaintBox;
    procedure FormCreate(Sender: TObject);
    procedure TreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure TreeViewDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure TreeViewEdited(Sender: TObject; Node: TTreeNode;
      var S: String);
    procedure NewClick(Sender: TObject);
    procedure NewSubClick(Sender: TObject);
    procedure DeleteClick(Sender: TObject);
    procedure LoadClick(Sender: TObject);
    procedure ApplyBtnClick(Sender: TObject);
    procedure HelpClick(Sender: TObject);
    procedure TextChange(Sender: TObject);
    procedure TextExit(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure AssignBtnClick(Sender: TObject);
    procedure TreeViewKeyPress(Sender: TObject; var Key: Char);
    procedure TreeViewEditing(Sender: TObject; Node: TTreeNode;
      var AllowEdit: Boolean);
    procedure ImagePaintPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ImagePaintMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TreeViewDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
  private
    ImgX, ImgY, SImgX, SImgY: Integer;
    procedure SetModified(Value: Boolean);
  public
    ActualTreeView: TTreeView;
    FModified: Boolean;
    procedure MenuClick(Sender: TObject);
    procedure PopulateMenu(AMenu: TPopupMenu; Images: TCustomImageList);
    procedure GetTreeViewList(AList: TStrings);
    procedure TreeMenuClick(Sender: TObject);
    procedure BuildAssignMenu;
    property Modified: Boolean read FModified write SetModified;
  end;

  procedure Register;

  function ShowTreeViewItemsDlg(ATreeView: TTreeView): Boolean;

var
  TreeViewItemsDlg: TTreeViewItemsDlg;

implementation

uses
  DsgnIntf;

{$R *.DFM}

type
  THETreeNodesEditor = class(TPropertyEditor)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
    function GetValue: string; override;
  end;

  THEEditTreeNodesEditor = class(TDefaultEditor)
  public
    procedure ExecuteVerb(Index : Integer); override;
    function GetVerb(Index : Integer): string; override;
    function GetVerbCount : Integer; override;
    procedure Edit; override;
  end;

//------------------------Registration------------------------------------------
procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(TTreeNodes), TTreeView, 'Items', THETreeNodesEditor);
  RegisterComponentEditor(TTreeView, THEEditTreeNodesEditor);
end;

//-----------------------THETreeNodesEditor-------------------------------------
procedure THEEditTreeNodesEditor.ExecuteVerb(Index: Integer);
begin
  if Index <> 0 then Exit;
  Edit;
end;

function THEEditTreeNodesEditor.GetVerb(Index: Integer): AnsiString;
begin
  Result := 'HE`s Items Editor...';
end;

function THEEditTreeNodesEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

procedure THEEditTreeNodesEditor.Edit;
begin
  if ShowTreeViewItemsDlg(TTreeView(Component)) then
    Designer.Modified;
end;

//-----------------------THETreeNodesEditor-------------------------------------
function THETreeNodesEditor.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog, paReadOnly]
end;

procedure THETreeNodesEditor.Edit;
begin
  inherited Edit;
  if ShowTreeViewItemsDlg(TTreeView(GetComponent(0))) then
    Designer.Modified;
end;

function THETreeNodesEditor.GetValue: string;
begin
  FmtStr(Result, '(%s)', [GetPropType^.Name]);
end;

//--------------
function ShowTreeViewItemsDlg(ATreeView: TTreeView): Boolean;
begin
  Result := False;
  if ATreeView = nil then Exit;

  with TTreeViewItemsDlg.Create(nil) do
  begin
    try
      ActualTreeView := ATreeView;
      Modified := False;
      TreeView.Items.Assign(ATreeView.Items);
      TreeView.Images := ATreeView.Images;
      TreeView.StateImages := ATreeView.StateImages;
      PopulateMenu(ImagesMenu, ATreeView.Images);
      PopulateMenu(SelectedMenu, ATreeView.Images);
      PopulateMenu(StateMenu, ATreeView.StateImages);
      BuildAssignMenu;
      ImageBtn.Enabled := (TreeView.Images <> nil);
      SelectedBtn.Enabled := (TreeView.Images <> nil);
      StateBtn.Enabled := (TreeView.StateImages <> nil);
      TreeView.SortType := ATreeView.SortType;
      TreeView.ShowLines := ATreeView.ShowLines;
      TreeView.ShowRoot := ATreeView.ShowRoot;
      TreeView.ShowButtons := ATreeView.ShowButtons;
      if TreeView.Items.Count > 0 then
        TreeView.Selected := TreeView.Items[0];
      if TreeMenu.Items.Count <= 0 then
        AssignBtn.Enabled := False;
      if Assigned(TreeView.Images) then
      begin
        ImgX := (ImagePaint.Width - TreeView.Images.Width) div 2;
        ImgY := (ImagePaint.Height - TreeView.Images.Height) div 2;
      end;
      if Assigned(TreeView.StateImages) then
      begin
        SImgX := (StatePaint.Width - TreeView.StateImages.Width) div 2;
        SImgY := (StatePaint.Height - TreeView.StateImages.Height) div 2;
      end;

      if (ShowModal = mrOK) and Modified then
        ATreeView.Items.Assign(TreeView.Items);
    finally
      Free;
    end;
  end;
end;

procedure TTreeViewItemsDlg.SetModified(Value: Boolean);
begin
  if not(FModified) and Value then
    FModified := True;
  ApplyBtn.Enabled := Value;
end;

procedure TTreeViewItemsDlg.PopulateMenu(AMenu: TPopupMenu; Images: TCustomImageList);
var
  i: Integer;
  FItem: TMenuItem;
  ModFactor: Integer;
begin
  //Populate with menuitems
  if not Assigned(Images) then Exit;

  ModFactor := 20;
  if Images.Height > 24 then
    ModFactor := 10;
  AMenu.Images := Images;
  for i := 0 to AMenu.Images.Count do
  begin
    FItem := TMenuItem.Create(AMenu.Items);
    FItem.Caption := IntToStr(i-1);
    FItem.OnClick := MenuClick;
    FItem.ImageIndex := i-1;
    FItem.RadioItem := True;
    AMenu.Items.Add(FItem);
    if ((i+1) mod ModFactor) = 0 then
      FItem.Break := mbBarBreak;
  end;
end;

procedure TTreeViewItemsDlg.MenuClick(Sender: TObject);
begin
  if TreeView.Selected = nil then Exit;

  if TMenuItem(Sender).GetParentMenu = ImagesMenu then
    TreeView.Selected.ImageIndex := TMenuItem(Sender).ImageIndex
  else if TMenuItem(Sender).GetParentMenu = SelectedMenu then
    TreeView.Selected.SelectedIndex := TMenuItem(Sender).ImageIndex
  else if TMenuItem(Sender).GetParentMenu = StateMenu then
    TreeView.Selected.StateIndex := TMenuItem(Sender).ImageIndex;
  TreeViewChange(Sender, TreeView.Selected);
end;

procedure TTreeViewItemsDlg.FormCreate(Sender: TObject);
begin
  FModified := False;
  //Modified := False;
end;

procedure TTreeViewItemsDlg.TreeViewChange(Sender: TObject; Node: TTreeNode);
begin
  Text.Text := Node.Text;
  ImagePaint.Tag := Node.ImageIndex;
  SelectedPaint.Tag := Node.SelectedIndex;
  StatePaint.Tag := Node.StateIndex;
  ImagePaint.Invalidate;
  SelectedPaint.Invalidate;
  StatePaint.Invalidate;
end;

procedure TTreeViewItemsDlg.TreeViewDragDrop(Sender, Source: TObject; X, Y: Integer);
begin
  TreeView.Selected.MoveTo(TreeView.DropTarget, naInsert);
end;

procedure TTreeViewItemsDlg.TreeViewEdited(Sender: TObject; Node: TTreeNode;
  var S: String);
begin
  //if Text.Text <> Node.Text then
    Modified := True;
  Text.Text := Node.Text;
end;

procedure TTreeViewItemsDlg.TreeViewEditing(Sender: TObject;
  Node: TTreeNode; var AllowEdit: Boolean);
begin
//  if (Node <> nil) then
//    Text.Text := Node.Text;
end;

procedure TTreeViewItemsDlg.NewClick(Sender: TObject);
var
  Node: TTreeNode;
begin
  if TreeView.Selected <> nil then
    Node := TreeView.Items.AddChild(TreeView.Selected.Parent, '')
  else
    Node := TreeView.Items.AddChild(nil, '');
  TreeView.Selected := Node;
  Node.EditText;
  Modified := True;
end;

procedure TTreeViewItemsDlg.NewSubClick(Sender: TObject);
var
  Node: TTreeNode;
begin
  if TreeView.Selected <> nil then
    Node := TreeView.Items.AddChild(TreeView.Selected, '')
  else
    Node := TreeView.Items.AddChild(nil, '');
  TreeView.Selected := Node;
  Node.EditText;
  Modified := True;
end;

procedure TTreeViewItemsDlg.DeleteClick(Sender: TObject);
begin
  if TreeView.Selected <> nil then
  begin
    TreeView.Selected.Delete;
    Modified := True;
  end;
end;

procedure TTreeViewItemsDlg.LoadClick(Sender: TObject);
begin
  if OpenDialog2.Execute then
  begin
    TreeView.LoadFromFile(OpenDialog2.FileName);
    Modified := True;
  end;
end;

procedure TTreeViewItemsDlg.ApplyBtnClick(Sender: TObject);
begin
  Modified := True;
  ActualTreeView.Items.Assign(TreeView.Items);
  Modified := False;
end;

procedure TTreeViewItemsDlg.HelpClick(Sender: TObject);
begin
  //
end;

procedure TTreeViewItemsDlg.TextChange(Sender: TObject);
begin
  if TreeView.Selected <> nil then
    TreeView.Selected.Text := Text.Text;
//  Modified := True;
end;

procedure TTreeViewItemsDlg.TextExit(Sender: TObject);
begin
  if TreeView.Selected <> nil then
  begin
    //if TreeView.Selected.Text <> Text.Text then
      Modified := True;
    TreeView.Selected.Text := Text.Text;
  end;
end;

procedure TTreeViewItemsDlg.Button1Click(Sender: TObject);
begin
  if OpenDialog2.Execute then
    TreeView.SaveToFile(OpenDialog2.FileName);
end;

procedure TTreeViewItemsDlg.ClearBtnClick(Sender: TObject);
begin
  TreeView.Items.Clear;
  Modified := True;
end;

procedure TTreeViewItemsDlg.SpeedButton1Click(Sender: TObject);
var
  CP: TPoint;
  AMenu: TPopupMenu;
  ImgIndex: Integer;
begin
  CP.x := TControl(Sender).Left + TControl(Sender).Width;
  CP.y := TControl(Sender).Top + TControl(Sender).Height;
  CP := TControl(Sender).Parent.ClientToScreen(CP);
  case TComponent(Sender).Tag of
    1: begin
         AMenu := SelectedMenu;
         ImgIndex := SelectedPaint.Tag;
       end;
    2: begin
         AMenu := StateMenu;
         ImgIndex := StatePaint.Tag;
       end;
  else
    begin
      AMenu := ImagesMenu;
      ImgIndex := ImagePaint.Tag;
    end;
  end;
  //Check selected ImageIndex menuitem
  if ImgIndex = -1 then
    AMenu.Items[0].Checked := True
  else if (ImgIndex+1 < AMenu.Items.Count) then
    AMenu.Items[ImgIndex+1].Checked := True;
  AMenu.PopUp(Cp.X, Cp.Y);
end;

procedure TTreeViewItemsDlg.TreeMenuClick(Sender: TObject);
begin
  TreeView.Items.Assign(TTreeView(TMenuItem(Sender).Tag).Items);
  TreeView.Invalidate;
  Modified := True;
end;

procedure TTreeViewItemsDlg.BuildAssignMenu;
var
  i: Integer;
  Comp: TComponent;
  FItem: TMenuItem;
begin
  if Assigned(ActualTreeView.Owner) then
  begin
    for i := 0 to ActualTreeView.Owner.ComponentCount-1 do
    begin
      Comp := ActualTreeView.Owner.Components[i];
      if (Comp is TTreeView) and (Comp <> ActualTreeView) then
      begin
        FItem := TMenuItem.Create(TreeMenu.Items);
        FItem.Caption := Comp.Name;
        FItem.OnClick := TreeMenuClick;
        FItem.Tag := Longint(Comp);
        TreeMenu.Items.Add(FItem);
        if (i mod 20) = 0 then
          FItem.Break := mbBarBreak;
      end;
    end;
  end;
end;

procedure TTreeViewItemsDlg.AssignBtnClick(Sender: TObject);
var
  CP: TPoint;
begin
  CP.x := TControl(Sender).Left + TControl(Sender).Width;
  CP.y := TControl(Sender).Top + TControl(Sender).Height;
  CP := TControl(Sender).Parent.ClientToScreen(CP);
  TreeMenu.PopUp(Cp.X, Cp.Y);
end;

procedure TTreeViewItemsDlg.GetTreeViewList(AList: TStrings);
var
  i: Integer;
  Comp: TComponent;
begin
  if Assigned(ActualTreeView.Owner) then
    for i := 0 to ActualTreeView.Owner.ComponentCount-1 do
    begin
      Comp := ActualTreeView.Owner.Components[i];
      if Comp is TTreeView then
        AList.AddObject(Comp.Name, Comp);
    end;
end;

procedure TTreeViewItemsDlg.TreeViewKeyPress(Sender: TObject;
  var Key: Char);
begin
  if (TreeView.Selected <> nil) and not(TreeView.IsEditing) then
    TreeView.Selected.EditText;
end;

procedure TTreeViewItemsDlg.ImagePaintPaint(Sender: TObject);
var
  Canv: TCanvas;
  PBox: TPaintBox;
  BRect: TRect;
begin
  PBox := TPaintBox(Sender);
  Canv := PBox.Canvas;
  Canv.Brush.Color := clBtnFace;
  Canv.Pen.Color := clBlack;
  BRect := Rect(0, 0, PBox.Width, PBox.Height);
  Canv.FillRect(BRect);
  DrawEdge(Canv.Handle, BRect, EDGE_RAISED, BF_RECT or BF_SOFT);
  if (PBox = ImagePaint) and Assigned(TreeView.Images) then
    TreeView.Images.Draw(Canv, ImgX, ImgY, PBox.Tag, True)
  else if (PBox = SelectedPaint) and Assigned(TreeView.Images) then
    TreeView.Images.Draw(Canv, ImgX, ImgY, PBox.Tag, True)
  else if (PBox = StatePaint) and Assigned(TreeView.StateImages) then
    TreeView.StateImages.Draw(Canv, SImgX, SImgY, PBox.Tag, True)
end;

procedure TTreeViewItemsDlg.FormResize(Sender: TObject);
begin
  Text.Width := Width - Text.Left - 12;
end;

procedure TTreeViewItemsDlg.ImagePaintMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  PBox: TPaintBox;
  SelNode: TTreeNode;
  AddSub: Integer;
begin
  if TreeView.Selected = nil then Exit;

  AddSub := 1;
  if Button = mbRight then AddSub := -1;

  SelNode := TreeView.Selected;
  PBox := TPaintBox(Sender);
  if PBox = ImagePaint then
    SelNode.ImageIndex := PBox.Tag + AddSub;
  if PBox = SelectedPaint then
    SelNode.SelectedIndex := PBox.Tag + AddSub;
  if PBox = StatePaint then
    SelNode.StateIndex := PBox.Tag + AddSub;

  if (PBox = StatePaint) and (TreeView.StateImages <> nil) and
     (SelNode.StateIndex >= TreeView.StateImages.Count) then
    SelNode.StateIndex := -1
  else if (TreeView.Images <> nil) then
    begin
      if (PBox = ImagePaint) and (SelNode.ImageIndex >= TreeView.Images.Count) then
        SelNode.ImageIndex := -1
      else if (SelNode.SelectedIndex >= TreeView.Images.Count) then
        SelNode.SelectedIndex := -1;
    end;
  if (PBox = StatePaint) and (TreeView.StateImages <> nil) and
     (SelNode.StateIndex < -1) then
    SelNode.StateIndex := TreeView.StateImages.Count - 1
  else if (TreeView.Images <> nil) then
    begin
      if (PBox = ImagePaint) and (SelNode.ImageIndex < -1) then
        SelNode.ImageIndex := TreeView.Images.Count - 1
      else if (SelNode.SelectedIndex < -1) then
        SelNode.SelectedIndex := TreeView.Images.Count - 1;
    end;
  TreeViewChange(Sender, SelNode);
end;

procedure TTreeViewItemsDlg.TreeViewDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := True;
end;

end.
